@@ -218,19 +218,6 @@ queue_free (LinkConnection *cnx)
 	cnx->priv->write_queue = NULL;
 }
 
-static void
-dispatch_callbacks_drop_lock (LinkConnection *cnx)
-{
-	GSList *callbacks;
-
-	callbacks = cnx->idle_broken_callbacks;
-	cnx->idle_broken_callbacks = NULL;
-
-	CNX_UNLOCK (cnx);
-	link_connection_emit_broken (cnx, callbacks);
-	CNX_LOCK (cnx);
-}
-
 /*
  * link_connection_class_state_changed:
  * @cnx: a #LinkConnection
@@ -303,9 +290,16 @@ link_connection_state_changed_T_R (LinkConnection      *cnx,
 
 			if (cnx->idle_broken_callbacks) {
 				if (!link_thread_io ()) {
+					GSList *callbacks;
+
 					d_printf ("Immediate broken callbacks at immediately\n");
-				
-					dispatch_callbacks_drop_lock (cnx);
+
+					callbacks = cnx->idle_broken_callbacks;
+					cnx->idle_broken_callbacks = NULL;
+
+					CNX_UNLOCK (cnx);
+					link_connection_emit_broken (cnx, callbacks);
+					CNX_LOCK (cnx);
 				} else {
 					d_printf ("Queuing broken callbacks at idle\n");
 
@@ -540,7 +534,7 @@ link_connection_do_initiate (LinkConnection        *cnx,
 	if (fcntl (fd, F_SETFD, FD_CLOEXEC) < 0)
 		goto out;
 
-	rv = connect (fd, saddr, saddr_len);
+	LINC_TEMP_FAILURE_RETRY (connect (fd, saddr, saddr_len), rv);
 	if (rv && errno != EINPROGRESS)
 		goto out;
 
@@ -587,14 +581,8 @@ link_connection_try_reconnect (LinkConnection *cnx)
 	d_printf ("Try for reconnection on %p: %d\n",
 		  cnx, cnx->inhibit_reconnect);
 
-	while (cnx->inhibit_reconnect) {
-		if (g_main_context_acquire (NULL)) {
-			d_printf ("Dispatch callbacks in 'main' (mainloop owning) thread\n");
-			dispatch_callbacks_drop_lock (cnx);
-			g_main_context_release (NULL);
-		} else 
-			link_wait ();
-	}
+	while (cnx->inhibit_reconnect)
+		link_wait ();
 
 	if (cnx->status != LINK_DISCONNECTED)
 		g_warning ("trying to re-connect connected cnx.");
@@ -739,7 +727,9 @@ link_connection_read (LinkConnection *cnx,
 			n = SSL_read (cnx->priv->ssl, buf, len);
 		else
 #endif
-			n = read (cnx->priv->fd, buf, len);
+			LINC_TEMP_FAILURE_RETRY (read (cnx->priv->fd, 
+							 buf, 
+							 len), n);
 
 		g_assert (n <= len);
 
@@ -863,8 +853,9 @@ write_data_T (LinkConnection *cnx, QueuedWrite *qw)
 				       qw->vecs->iov_len);
 		else
 #endif
-			n = writev (cnx->priv->fd, qw->vecs,
-				    MIN (qw->nvecs, LINK_IOV_MAX));
+			LINC_TEMP_FAILURE_RETRY (writev (cnx->priv->fd,
+							     qw->vecs,
+							     MIN (qw->nvecs, LINK_IOV_MAX)), n);
 
 		d_printf ("wrote %d bytes\n", n);
 
